
/*
 * Copyright (c) 2005, Arnaud KLEIN
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */ 
 
#include "LogEntryFormat.h" 
#include <sys/types.h>
#include <sys/socket.h>
#include <time.h>
#include <netinet/in.h>
#include <arpa/inet.h>


#define STATIC_BUFFER_LEN	128


/* Constructor */
cLogEntryFormat::cLogEntryFormat(const cLogEntry& entry) : m_entry(entry), m_time(), m_ifName(), m_ruleNumber(), m_action(), m_direction(), m_protocol(), m_destAddress(), m_srcAddress(), m_destPort(), m_srcPort()
{
}


/* Format */
void cLogEntryFormat::Format()
{
	// Time
	FormatTime();
	
	// Pflog header
	if ((m_entry.GetEntryDataFlags() & cLogEntry::ENTRYDATA_PFLOG) == cLogEntry::ENTRYDATA_PFLOG)
	{
		m_ifName=m_entry.GetIfName();
		FormatRuleNumber();
		FormatAction();
		FormatDirection();
		FormatProtocol();
	}

	// IP header
	if ((m_entry.GetEntryDataFlags() & cLogEntry::ENTRYDATA_IP) == cLogEntry::ENTRYDATA_IP)
		FormatAddresses();

	// TCP or UDP header
	if ((m_entry.GetEntryDataFlags() & (cLogEntry::ENTRYDATA_TCP | cLogEntry::ENTRYDATA_UDP)) != 0)
		FormatPorts();
}


/* Format time */
void cLogEntryFormat::FormatTime()
{
	time_t t=m_entry.GetTime().tv_sec;
	
	// Convert struct timeval to struct tm
	struct tm* localTime=localtime(&t);
	if (localTime)
	{
	
		// Convert time to string
		char strBuffer[STATIC_BUFFER_LEN];
		if (strftime(strBuffer, STATIC_BUFFER_LEN, "%F %T", localTime) > 0)
			m_time=strBuffer;
	}
}


/* Format rule number */
void cLogEntryFormat::FormatRuleNumber()
{
	char strBuffer[STATIC_BUFFER_LEN];
	if (snprintf(strBuffer, STATIC_BUFFER_LEN, "%u", m_entry.GetRuleNumber()) > 0)
		m_ruleNumber=strBuffer;
}


/* Format action */
void cLogEntryFormat::FormatAction()
{
	if (m_entry.GetAction() == PF_PASS)
		m_action="pass";
	else if (m_entry.GetAction() == PF_DROP)
		m_action="drop";
	else
	{
		char strBuffer[STATIC_BUFFER_LEN];

		if (snprintf(strBuffer, STATIC_BUFFER_LEN, "(%x)", m_entry.GetAction()) > 0)
			m_action=strBuffer;
	}
}


/* Format direction */
void cLogEntryFormat::FormatDirection()
{
	if (m_entry.GetDirection() == PF_IN)
		m_direction="in";
	else if (m_entry.GetDirection() == PF_OUT)
		m_direction="out";
	else if (m_entry.GetDirection() == PF_INOUT)
		m_direction="in-out";
}


/* Format protocol */
void cLogEntryFormat::FormatProtocol()
{
	switch(m_entry.GetProtocol())
	{
		case IPPROTO_IP:
		{
			m_protocol="ip";
			break;
		}
		
		case IPPROTO_ICMP:
		{
			m_protocol="icmp";
			break;
		}
		
		case IPPROTO_TCP:
		{
			m_protocol="tcp";
			break;
		}
		
		case IPPROTO_UDP:
		{
			m_protocol="udp";
			break;
		}
		
		case IPPROTO_RAW:
		{
			m_protocol="raw";
			break;
		}
		
		default:
		{
			char strBuffer[STATIC_BUFFER_LEN];
			if (snprintf(strBuffer, STATIC_BUFFER_LEN, "(%x)", m_entry.GetProtocol()) > 0)
				m_protocol=strBuffer;
		}
	}
}


/* Format source and destination addresses */
void cLogEntryFormat::FormatAddresses()
{
	const char* strBuffer=NULL;
	
	strBuffer=inet_ntoa(m_entry.GetSrcAddress());
	if (strBuffer)
		m_srcAddress=strBuffer;

	strBuffer=inet_ntoa(m_entry.GetDestAddress());
	if (strBuffer)
		m_destAddress=strBuffer;
}


/* Format source and destination ports */
void cLogEntryFormat::FormatPorts()
{
	char strBuffer[STATIC_BUFFER_LEN];
	
	if (snprintf(strBuffer, STATIC_BUFFER_LEN, "%u", m_entry.GetSrcPort()) > 0)
		m_srcPort=strBuffer;

	if (snprintf(strBuffer, STATIC_BUFFER_LEN, "%u", m_entry.GetDestPort()) > 0)
		m_destPort=strBuffer;
}


