
/*
 * Copyright (c) 2005, Arnaud KLEIN
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */ 
 
#include "LogExport.h" 
#include "LogEntryFormat.h"
#include "pflogx.h"
#include <errno.h>
#include <fstream>
#include <string.h>


/* Constructor */
cLogExport::cLogExport(const string& file, const vector<cLogEntry>& vectLogEntries) : m_file(file), m_vectLogEntries(vectLogEntries), m_outputStream(NULL), m_errorString() 
{
}


/* Export */
bool cLogExport::Export()
{
	bool bHasError=false;

	if (OpenFile())
	{
		// Write header
		if (WriteHeader())
		{
			// Write entries
			for(vector<cLogEntry>::const_iterator it=m_vectLogEntries.begin(); !bHasError && (it!=m_vectLogEntries.end()); it++)
				bHasError=!WriteEntry(*it);
		
			if (!bHasError)
			{
				// Write footer
				if (!WriteFooter())
					bHasError=true;
			}
		}
		else
			bHasError=true;
	

		CloseFile();
	}
	else
		bHasError=true;

	return !bHasError;
}


/* Open output file */
bool cLogExport::OpenFile()
{
	if (OutputOnStdOut())
		m_outputStream=&cout;
	else
	{
		// Construct new ofstream object and open file for writing
		m_outputStream=new (nothrow) ofstream(m_file.c_str());
		
		if (!m_outputStream || !static_cast<ofstream*>(m_outputStream)->is_open())
		{
			m_errorString="Could not open output file (" + GetSystemError() + ")";
			return false;
		}
	}
	
	return true;
}
	

/* Close output file */
bool cLogExport::CloseFile()
{
	if (!m_outputStream)
	{
		m_errorString="Output file not opened";
		return false;
	}
		
	if (OutputOnStdOut())
		m_outputStream=NULL;
	else
	{
		if (static_cast<ofstream*>(m_outputStream)->is_open())
			static_cast<ofstream*>(m_outputStream)->close();

		delete m_outputStream;
		m_outputStream=NULL;
	}

	return true;
}


/* Write a string to outfile file */
bool cLogExport::WriteString(const string& str)
{
	m_outputStream->write(str.c_str(), str.length());
	
	if (m_outputStream->fail())
	{
		m_errorString="Write error (" + GetSystemError() + ")";
		return false;
	}
	else
		return true;
}


/* Write file header */
bool cLogExport::WriteHeader()
{
	string strHeader="<?xml version=\"1.0\"?>\n<pflogx version=\"" + string(PFLOGX_VERSION) + "\" >\n<logs>\n";
	return WriteString(strHeader);
}


/* Write file footer */
bool cLogExport::WriteFooter()
{
	string strFooter="</logs>\n</pflogx>\n";
	return WriteString(strFooter);
}


/* Write entry */
bool cLogExport::WriteEntry(const cLogEntry& entry)
{
	cLogEntryFormat format(entry);
	format.Format();
	
	string strEntry="<log date=\"" + format.GetTime() + "\" " + 
	 				"if=\"" + format.GetIfName() + "\" " +
					"action=\"" + format.GetAction() + "\" " +
					"rule=\"" + format.GetRuleNumber() + "\" " +
					"direction=\"" + format.GetDirection() + "\" " +
					"protocol=\"" + format.GetProtocol() + "\" " +
					"src_adr=\"" + format.GetSrcAddress() + "\" " +
					"src_port=\"" + format.GetSrcPort() + "\" " +
					"dest_adr=\"" + format.GetDestAddress() + "\" " +
					"dest_port=\"" + format.GetDestPort() + "\" " +
					"/>\n";

	return WriteString(strEntry);
}


/* Retrieve last system error message */
const string cLogExport::GetSystemError() const
{
#define BUFFER_SIZE 512	

	string strError;
	char strBuffer[BUFFER_SIZE];

	if (strerror_r(errno, strBuffer, BUFFER_SIZE) == 0)
		strError=strBuffer;

	return strError;

#undef BUFFER_SIZE
} 

