
/*
 * Copyright (c) 2005, Arnaud KLEIN
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */ 


#include <iostream>
#include <stdlib.h>
#include "pflogx.h"
#include "CmdLineParser.h"
#include "EntriesFilter.h"
#include "LogEntry.h"
#include "LogExport.h"
#include "LogParser.h"
#include "Options.h"


/* Display program usage */
void usage(const char* progName)
{
	cout<<"pflogx - Export packet filter logs to XML"<<endl;
	cout<<"Copyright (c) 2005, Arnaud KLEIN "<<endl;
	cout<<endl;
	cout<<"Usage:"<<endl;
	cout<<"    "<<progName<<" [OPTIONS]"<<endl;
	cout<<endl;
	cout<<"Options:"<<endl;
	cout<<"    -i <input_file>: Packet filter logfile (Default: standard input)"<<endl;
	cout<<"    -o <output_file>: Output XML file (Default: standard output)"<<endl;
	cout<<"    -a <filter>: Define filter on action (Default: no filter)"<<endl;
	cout<<"    -d <filter>: Define filter on direction (Default: no filter)"<<endl;
	cout<<"    -p <filter>: Define filter on protocol (Default: no filter)"<<endl;
	cout<<"    -r: Sort events in reverse chronological order"<<endl;
	cout<<"    -e: Show erroneous packets"<<endl;
	cout<<"    -h: Display program usage"<<endl;
	cout<<"    -v: Display program version"<<endl;
}


/* Display program version */
void version()
{
	cout<<"pflogx - Export packet filter logs to XML"<<endl;
	cout<<"Copyright (c) 2005, Arnaud KLEIN "<<endl;
	cout<<endl;
	cout<<"Version "<<PFLOGX_VERSION<<endl;
}


/* main */
int main(int argc, char** argv)
{
	int nRet=EXIT_FAILURE;

	cCmdLineParser::Initialize();


	// Parse command line
	cCmdLineParser cmdLineParser(argc, argv);
	enum cCmdLineParser::eParseRet parseRet=cmdLineParser.Parse();
	
	if (parseRet == cCmdLineParser::PARSE_NO_ERROR)
	{
		// Get options
		const cOptions& options=cmdLineParser.GetOptions();
		
		// Parse input file
		cLogParser logParser;
		vector<cLogEntry> vectLogEntries;
		vector<pair<int, enum cLogParser::eParsePacketError> > vectLogErrors;
		
		if (logParser.ParseLog(options.GetInputFile(), vectLogEntries, vectLogErrors))
		{
			// Display erroneous packets
			if (options.GetDisplayErrorsPackets())
			{
				for(vector<pair<int, enum cLogParser::eParsePacketError> >::const_iterator it=vectLogErrors.begin(); it!=vectLogErrors.end(); it++)
				{
					int packet_num=(*it).first;
					const string packet_error=logParser.GetParsePacketErrorMessage((*it).second);
				
					cerr<<"Error in packet "<<packet_num<<": "<<packet_error<<"."<<endl;
				}
			}
			
			// Filter entries
			cEntriesFilter entriesFilter(vectLogEntries, options); 
			entriesFilter.ApplyFilters();

			// Export packets
			cLogExport logExport(options.GetOutputFile(), entriesFilter.GetEntries());
			
			if (logExport.Export())
			{
				// Success
				nRet=EXIT_SUCCESS;
			}
			else
			{
				cerr<<"Export error: "<<logExport.GetErrorString()<<"."<<endl;
			}
		}
		else
		{
			cerr<<"Libpcap error: "<<logParser.GetPcapErrorMessage()<<"."<<endl;
		}
	}
	else if (parseRet == cCmdLineParser::PARSE_HELP_DISPLAY)
	{
		usage(argv[0]);
		nRet=EXIT_SUCCESS;
	}
	else if (parseRet == cCmdLineParser::PARSE_VERSION_DISPLAY)
	{
		version();
		nRet=EXIT_SUCCESS;
	}
	else if (parseRet == cCmdLineParser::PARSE_ERROR)
	{
		cerr<<"Option error: "<<cmdLineParser.GetErrorString()<<"."<<endl;
	}
	
	return nRet;
}

